//  Copyright 2019 Kakao Corp.
//
//  Licensed under the Apache License, Version 2.0 (the "License");
//  you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
//  Unless required by applicable law or agreed to in writing, software
//  distributed under the License is distributed on an "AS IS" BASIS,
//  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//  See the License for the specific language governing permissions and
//  limitations under the License.

import UIKit
import Foundation
import KakaoSDKCommon
import KakaoSDKAuth
import KakaoSDKCert

import CryptoKit

///:nodoc: 이용기관 테스트용 mock-server
public class CertDemoHost {
    var _domain: String
    
    public init(domain: String = "zert-mock.dev.onkakao.net") {
        _domain = domain
    }
    
    public func domain() -> String {
        return _domain
    }
}

///nodoc:
import Alamofire

///nodoc: //demo 서버 api
public class CertDemoApi {
    public static let shared = CertDemoApi()
    
    static let txIdKey = "com.kakao.sdk.k2220.txId"
    
    private var _txId: String? = nil
    
    public var demoHost: CertDemoHost
    
    public init() {
        _txId = UserDefaults.standard.string(forKey: CertDemoApi.txIdKey)
        demoHost = CertDemoHost()
    }
}

extension CertDemoApi {
    public func txId() -> String? {        
        return _txId
    }
    
    public func saveTxId(_ txId: String?) {
        _txId = txId
        UserDefaults.standard.setValue(_txId, forKey:CertDemoApi.txIdKey)
        UserDefaults.standard.synchronize()
        
    }
    
    public func deleteTxId() {
        UserDefaults.standard.removeObject(forKey: CertDemoApi.txIdKey)
        _txId = nil
    }
}
   
extension CertDemoApi {
    ///:nodoc:
    public func demoLogin(base64encodedDerPublicKey:String? = nil,
                                completion: @escaping (String?, Error?) -> Void) {
        guard let base64encodedDerPublicKey = base64encodedDerPublicKey else { return }

        let param: [String:String] = [
            "public_key":base64encodedDerPublicKey
        ]

        let url = "https://\(CertDemoApi.shared.demoHost.domain())\(Paths.demoLogin)"
        //TODO:릴리즈시 print로 교체
        SdkLog.i("request: \n method: \(HTTPMethod.post)\n url:\(url)\n parameters: \(String(describing: param)) \n\n")

        AF.request(url, method: .post, parameters: param)
          .responseData(completionHandler: { response in
              
              //TODO:릴리즈시 print로 교체
              SdkLog.i("response:\n \(String(describing: response))\n\n" )
              
              if let error = response.error {
                  completion(nil, error)
              }
              else {
                  guard let data = response.data else {
                      completion(nil, CertDemoError(msg:"data is nil"))
                      return
                  }
                  
                  if let error = CertDemoError(data: data) {
                      completion(nil, error)
                  }
                  else {
                      if let certResponse = try? SdkJSONDecoder.custom.decode(CertDemoResponse.self, from: data) {
                          print("response: \(certResponse)")
                          completion(certResponse.txId, nil)
                      }
                  }
              }
          })
    }

    ///:nodoc:
    public func demoVerify(txId: String,
                           appUserId: Int64,
                           completion: @escaping (CertDemoResponse?, Error?) -> Void) {
        let param: [String:Any] = [
            "tx_id":txId,
            "app_user_id":appUserId
        ]

        let url = "https://\(CertDemoApi.shared.demoHost.domain())\(Paths.demoVerify)"
        //TODO:릴리즈시 print로 교체
        SdkLog.i("request: \n method: \(HTTPMethod.post)\n url:\(url)\n parameters: \(String(describing: param)) \n\n")

        AF.request(url, method: .post, parameters: param)
          .responseData(completionHandler: { response in
              
              //TODO:릴리즈시 print로 교체
              SdkLog.i("response:\n \(String(describing: response))\n\n" )
              
              if let error = response.error {
                  completion(nil, error)
              }
              else {
                  guard let data = response.data else {
                      completion(nil, CertDemoError(msg:"data is nil"))
                      return
                  }
                  
                  if let error = CertDemoError(data: data) {
                      completion(nil, error)
                  }
                  else {
                      if let certResponse = try? SdkJSONDecoder.custom.decode(CertDemoResponse.self, from: data) {
                          print("response: \(certResponse)")
                          completion(certResponse, nil)
                      }
                  }
              }
          })
    }

    ///:nodoc:
    public func demoSign(txId:String, data:String, signature:String, completion: @escaping (CertDemoResponse?, Error?) -> Void) {
        let param: [String:String] = [
            "tx_id":txId,
            "data": data,
            "signature":signature
        ]

        let url = "https://\(CertDemoApi.shared.demoHost.domain())\(Paths.demoSign)"
        //TODO:릴리즈시 print로 교체
        SdkLog.i("request: \n method: \(HTTPMethod.post)\n url:\(url)\n parameters: \(String(describing: param)) \n\n")

        AF.request(url, method: .post, parameters: param)
          .responseData(completionHandler: { response in
              
              //TODO:릴리즈시 print로 교체
              SdkLog.i("response:\n \(String(describing: response))\n\n" )
              
              if let error = response.error {
                  completion(nil, error)
              }
              else {
                  guard let data = response.data else {
                      completion(nil, CertDemoError(msg:"data is nil"))
                      return
                  }
                  
                  if let error = CertDemoError(data: data) {
                      completion(nil, error)
                  }
                  else {
                      if let certResponse = try? SdkJSONDecoder.custom.decode(CertDemoResponse.self, from: data) {
                          print("response: \(certResponse)")
                          completion(certResponse, nil)
                      }
                  }
              }
          })
    }
}

public struct CertDemoResponse : Codable {
    public let code: String?
    public let msg: String?
    public let txId: String?
}

public struct CertDemoError : Error {
    public let code: String?
    public let msg: String?
    
    public init?(data:Data) {
        guard let certDemoResponse = try? SdkJSONDecoder.custom.decode(CertDemoResponse.self, from: data) else {
            self.code = "E----"
            self.msg = "can't parse response"
            return
        }
        if certDemoResponse.code == "E0000" { return nil }
        
        self.code = certDemoResponse.code
        self.msg = certDemoResponse.msg
    }
    
    public init(msg:String? = nil) {
        self.code = "E----"
        self.msg = msg ?? "unknown"
    }
}
